
/*	EXAMPLE 1. PIEPHO-WILLIAMS-MADDEN. 2012. BIOMETRICS 68: 1269-1279. 
	Smoking cessation data of Hasselblad (1998). Used by Lu & Ades(2006) and Lu et al.(2011). 
	There are 24 studies and 4 treatments (although all treatments are not in all studies).

	DEMONSTRATES THE ANALYSIS OF INDIVIDUAL PATIENT DATA (IPD), AND ALSO ANALYSIS OF 
	SUMMARY RESULTS (EFFECT SIZE: ESTIMATED EXPECTED VALUES OR ESTIMATED CONTRASTS WITH
	BASELINE) IN A 2-STAGE ANALYSIS. 
	SHOWS THE EQUIVALENCE (UNDER CERTAIN CONDITIONS) OF AN ANALYSIS BASED ON CONTRASTS
	WITH BASELINE AND AN ANALYSIS BASED ON THE EXPECTED VALUE PER TREATMENT (WITH SUMMARY 
	MEASURES AND USE OF REML).

	EXAMPLE ALSO SHOWS OUR APPROACH TO AN ANALYSIS OF CONSISTENCY, BASED ON A MIXED MODEL.
*/

title 'Ex. 1: Piepho-Williams-Madden(smoking data of Hasselblad, used by Lu&Ades, Lu et al.)';
data smoking;
input study trt$ m n;
datalines;
1  a 9 140
1  c 23 140
1  d 10 138
2  b 11 78
2  c 12 85 
2  d 29 170
3  a 79 702
3  b 77 694
4  a 18 671
4  b 21 535
5  a 8 116
5  b 19 146
6  a 75 731
6  c 363 714
7  a 2 106
7  c 9 205
8  a 58 549
8  c 237 1561
9  a 0 33
9  c 9 48
10 a 3 100
10 c 31 98
11 a 1 31
11 c 26 95
12 a 6 39
12 c 17 77
13 a 95 1107
13 c 134 1031
14 a 15 187
14 c 35 504
15 a 78 584
15 c 73 675
16 a 69 1177
16 c 54 888
17 a 64 642
17 c 107 761
18 a 5 62
18 c 8 90
19 a 20 234
19 c 34 237
20 a 0 20
20 d 9 20
21 b 20 49
21 c 16 43
22 b 7 66
22 d 32 127
23 c 12 76
23 d 20 74
24 c 9 55
24 d 3 26
;
run;

/*	Do a lot of data processing to define groups of studies with the same treatments, and also
	to define contrasts from the baseline. Later in this file, more processing will be done
	in order to work on the summary data. Remnants from some alternative approaches or 
	experimental approaches may still exist in the data steps (not used). In some of these
	remnants, different quadrature points were investigated, and maybe other options.
	
	
	For this example, one needs the x_ab, x_ac, and x_ad variables for the analysis of 
	Lu & Ades. One needs the group variable for our analysis of consistency. Some other created
	variables may not be used in this example (such as t_b, etc., and w_ab, w_abc, etc.).
*/

data a; set smoking;
if trt='a' then trt='za';  /*make a the last category 
                            to get same restriction on 
                            basic effects as Lu&Andes*/
/*Group studies according to set of treatments tested. This is for our approach to consistency.*/
if study in (1) then group=1;							*<--"group" is for categories of studies;
if study in (2) then group=2;							*<--group depends on the treatments in the study;	
if study in (3,4,5) then group=3;
if study in (6,7,8,9,10,11,12,13,14,15,16,17,18,19) then group=4;
if study in (20) then group=5;
if study in (21) then group=6;
if study in (22) then group=7;
if study in (23,24) then group=8;
/*dummy variables for inconsistency for our approach*/
w_BC=0; if study in (2, 21)     then do; if trt='c' then w_BC=1; *if trt='c' then w_BC=-1; end;
w_BD=0; if study in (2, 22)     then do; if trt='d' then w_BD=1; *if trt='d' then w_BD=-1; end;
w_CD=0; if study in (1,2,23,24) then do; if trt='d' then w_CD=1; *if trt='d' then w_CD=-1; end;
/*code for analysis according to Lu and Ades 2006*/
if study in (1,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20) then baseline='za';
if study in (2,21,22) then baseline='b';
if study in (23,24) then baseline='c';
*if study in (6,7,8,9,10,11,12,13,14,15) then baseline='c'; 
/*^^(above) modify (take out *) to see if this choice of baseline affects the
	conditional analysis. Change is arbitrary*/

x_ab=0; x_ac=0; x_ad=0;     /*dummies for deltas*/		*<--This parameterization is for contrasts in each study;
w_abc=0; w_abd=0; w_acd=0;								*<--Could be used below for CONSISTENCY estimation;
base_dummy=1;
t_a=0; t_b=0; t_c=0; t_d=0;								*<--not used right now;
if baseline='za' then do;
  t_a=-1;
  if trt='za' then do; base_dummy=0; t_a=0; end;		*<--Baseline can be different for different studies;
  if trt='b'  then do; x_ab=1; t_b=1; end;
  if trt='c'  then do; x_ac=1; t_c=1; end;
  if trt='d'  then do; x_ad=1; t_d=1; end;
end;
if baseline='b' then do;
  t_b=-1;
  if trt='za' then do; x_ab=-1; t_a=1; end;
  if trt='b' then do; base_dummy=0; t_b=0; end;
  if trt='c'  then do; x_ac=1; x_ab=-1; w_abc=1; t_c=1; end;
  if trt='d'  then do; x_ad=1; x_ab=-1; w_abd=1; t_d=1; end;
end;
if baseline='c' then do;
  t_c=-1;
  if trt='za' then do; x_ac=-1; t_a=1; end;
  if trt='b'  then do; x_ac=-1; x_ab=1; w_abc=-1; t_b=1; end;
  if trt='c' then do; base_dummy=0; t_c=0; end;
  if trt='d'  then do; x_ad=1; x_ac=-1; w_acd=1; t_d=1; end;
end;
run;

proc sort data=a out=asort;
by study trt;

proc print data=asort;
title2 'look at the data file';
var study group trt baseline m n x_ab x_ac x_ad;
run;

/*	Frequentist version(s) of the fixed (study and trt) effects model, corresponding to results in 
	Table 2 of Lu & Ades (i.e., fixed for study and trt, random for trt*study). As discussed in
	our paper, the parameterization in our model is easier to use than the approach in 
	Lu and Ades. We call our model the unconditional model. 
	
	Note that there are several equivalent
	ways of writing some of the SAS statements. For instance, the random statement in the
	following model fits could be written as:
		random int / sub=study*trt;

	Sometimes we use Type I hypotheses (htype=1), where the default is Type III. Using the 
	default would not affect the trt result (since it is given last) for the unconditional
	model. 

	The denominator df can be specified in different ways. Often, we use ddfm=none in GLIMMIX,
	which essentially means that d.d.f is infinity. One then gets usual Wald statistics.
	Although redundant with this ddfm, we usually ask for the display of the Wald statistic
	by using the chisq option in the model statement. One could also use the Kenward-Roger 
	ddf method (add ddfm=KR as	an option on the model statement. 

	We generally use Gauss-Hermite Quadrature (method=quad) in model fitting to approximate
	the marginal log-likelihood. Sometimes we specifiy the number of quadrature points (with a
	qpoints= option), and sometimes we allow the number to be determined adaptively.
	The choice here for number of quadrature points does not necessarily agree with what
	was used in the paper.
*/

proc glimmix data=a maxopt=100 method=quad(qpoints=3);	*<--Used in our paper;
title2 'our 2-way mixed model, homogeneous variances';
title3 'IPD analysis, with fixed effects of study and trt, random trt*study';
class study trt;
model m/n = study trt / ddfm=none solution chisq;
random trt / sub=study;
lsmeans trt / pdiff lines;
estimate 'B-A' trt 1 0 0 -1;	*<---"a" is really last (as "za") --> order: b c d za;
estimate 'C-A' trt 0 1 0 -1;
estimate 'D-A' trt 0 0 1 -1;
run;
*^--Note that the complex data processing in the above data step was not need for 
	fitting this unconditinal model.;
*^--Above, ddfm=none gives infinity for denominator df. Not required. ;

proc glimmix data=a maxopt=100 method=quad(qpoints=1);	*<--used a little in paper;
title2 'our 2-way mixed model, UNEQUAL variances';
title3 'IPD analysis, with fixed effects of study and trt, random trt*study';
class study trt;
model m/n= study trt / ddfm=none solution chisq;
random trt / sub=study group=trt;
lsmeans trt / pdiff lines;
run;

proc glimmix data=a maxopt=100 method=quad(qpoints=6);
title2 'our 2-way mixed model, FA1(1) {factor analytic}';
title3 'IPD analysis, with fixed effects of study and trt, random trt*study';
class study trt; 
model m/n = study trt / ddfm=none solution chisq;
random trt / sub=study type=fa1(1);
lsmeans trt / pdiff lines;
run;

proc glimmix data=a maxopt=100 method=quad(qpoints=3);
title2 'our 2-way model with only FIXED effects';
title3 'IPD analysis, fixed study and trt -- No random study*trt effects';
class study trt;
model m/n= study trt / ddfm=none  solution chisq;
lsmeans trt/pdiff lines;
run;

/*mixed model with random study effects*/
proc glimmix data=a maxopt=100 method=quad(qpoints=3);
title2 'our model with random study effects, homogeneous variances';
title3 'IPD analysis, with fixed effects of trt, random effects of study and study*trt';
class study trt group;
model m/n = trt / ddfm=none solution chisq;
random int / sub=study;
random int / sub=study*trt;
lsmeans trt / pdiff lines;
run;


*---Our approach to consistency analysis with a mixed model -- based on expected values 
	per treatment. Include fixed-effects terms for group type (group) and the
	interaction of group type and treatment (trt*group). The group factor was defined
	above in a data step (it depends on the treatments in the study). ;

/*model with fixed study effects and consistency stuff*/
proc glimmix data=a maxopt=100 method=quad(qpoints=3);
title2 'our model with fixed study effects,  w/ inconsistency group  & inter. effects';
title3 'IPD analysis -- Test of trt*group for INCONSISTENCY';
class study trt group;
model m/n = group study trt trt*group / ddfm=none htype=1 solution chisq;	
*^--Often good to use type I hypotheses for the inconsistency testing.;
*^--Use results mainly for the _last_ term listed (trt*group) for inconsistency.;
random int / sub=study*trt;	
run;

/*model with fixed study effects and consistency stuff, _only_ FIXED EFFECTS*/
proc glimmix data=a maxopt=100 method=quad;
title2 'our FIXED EFF. model, with fixed study effects, w/ incons. group & inter. effects';
title3 'IPD analysis (no random study effects), test of trt*group for inconsistency ';
class study trt group;
model m/n = group study trt trt*group / ddfm=none htype=1 solution chisq;
run;

/* NOW DO MULTIPLE-TREATMENT META-ANALYSIS WITH THE CONDITIONAL MODEL OF LU & ADES (AND COLLEAGUES) */

/*	This approach requires the use of variables defined above for contrasts with baseline (e.g, x_ab).

	Make the 'structure' file for the among-study variance-covariance matrix, G, 
	used for the Lu-Ades or Lu et al. parameterization of the mixed model (what we call the 
	conditional model). Model based on the contrasts with baseline for each study. 
	Because the baseline is used in 
	multiple contrasts (when there are multiple treatments), the contrasts are correlated. For the
	homogenous case (what we are using here), the covariance is half the value of the variance, by definition.
	Thus, the actual between-study covariance matrix (for a given study) is "var*L3",
	where "var" is the variance _parameter_ (estimated from the data), and L3 is the matrix 
	giving the structure to the var-cov matrix. The syntax of the RANDOM statement in GLIMMIX, with the
	TYPE=LIN(1) and LDATA=L3 options, result in the estimate of "var" parameter. 
*/
*---L3 is the 'structure' for the covariance matrix of contrasts for each study. 
	Use with LDATA= option below. Because analysis is based on contrasts with
	baseline, the dimension of L3 is one less than the number of treatments
	(including baseline);
data L3;		
input parm row col1-col3;
datalines;
1 1 2 1 1
1 2 1 2 1
1 3 1 1 2
;

title2 'look at data and treatment-effect (contrast) variables for the conditional model';
proc print data=a;
var study trt m n x_ab x_ac x_ad baseline;run;

*---This is the Lu & Ades model with original baseline -- frequentist version.  The direct
	contrast parameters (known as BASIC parameters) are the coefficients of x_ab, x_ac, and x_ad. 
	Indirect parameters (known as FUNCTIONAL parameters) are the differences of the 
	basic parameters (in ESTIMATE statements).;

proc glimmix data=a maxopt=100 method=quad(qpoints=3);
title2 'Lu&Ades conditional model, contrasts with original baseline, frequentist version';
title3 'fixed study, random study*"trt" (u~_ik), homogeneous';
title4 'see relevant table 2 (Bayesian) results in THEIR paper';
class study trt;
model m/n = study x_ab x_ac x_ad
	/ ddfm=none htype=3 solution chisq noint;				*<--Need the x_ variables, study class, & NO INTERCEPT;
random x_ab x_ac x_ad / sub=study type=lin(1) ldata=L3;		*<--structure for G var-cov matrix is in file L3;
estimate 'd_BC' x_ac 1 x_ab -1;								*<--Get indirect parameters from the direct ones;
estimate 'd_BD' x_ad 1 x_ab -1;
estimate 'd_CD' x_ad 1 x_ac -1;
run;
*^--For this parameterization, the tests of x_ab, etc., will depend on whether type I or III is
	used for hypotheses. However, the estimates of the parameters and their SEs will not depend
	on the hypothesis type. Do not see a logical order for the three x_ variables, so type III
	could be best (?);

/*fixed effects only*/
proc glimmix data=a maxopt=100 method=quad;
title2 'Lu&Ades conditional model, contrasts with original baseline';
title3 'FIX.EFF. -- consistent model of Lu&Ades (fixed study, x_ab, etc.), no u~_ik';
title4 'see relevant table 2 results in THEIR paper';
class study trt;
model m/n = study x_ab x_ac x_ad
	/ ddfm=none htype=3 solution chisq;
estimate 'd_BC' x_ac 1 x_ab -1;
estimate 'd_BD' x_ad 1 x_ab -1;
estimate 'd_CD' x_ad 1 x_ac -1;
run;


*---Now do some tests of consistency with conditional model (x_ab, etc.), but use our
	coding of study group and group*trt interaction.;
proc glimmix data=a maxopt=100 method=quad(qpoints=7);
title2 'Lu et al. cond. model, orig. baseline, fixed study, random study*"trt" (u~_ik), homog.';
title3 'inconsis. testing (but with _OUR_ group and group*trt approach)';
title4 'this is similar to the approach in Lu et al., but we use frequentist approach';
class study trt group;
model m/n = group study x_ab x_ac x_ad group*trt
	/ ddfm=none htype=1 solution chisq;					*<--test of group*trt for inconsistency;
random x_ab x_ac x_ad / sub=study type=lin(1) ldata=L3;	*<--see above for meaning of the random statement;
run;
*^--Here we use type I testing, because one should focus on the last term in the model
	(grout*trt) to test consistency.;

proc glimmix data=a maxopt=100 method=quad(qpoints=7);
title2 'Lu et al. cond. model, orig. baseline contrasts, fixed study';
title3 'FIXED EFF. inconsistency testing (with _OUR_ group and group*"trt" approach)';
class study trt group;
model m/n= group study x_ab x_ac x_ad group*trt
	/ ddfm=none htype=1 solution chisq;
run;



/*
*---Now do the frequentist version of Lu & Ades analysis of consistency, using their
	conditional model. This approach requires the inconsistency terms w_abc, etc. These
	are determined in the above data step, based on the treatment makeup of each
	study. This is a very tricky step, and one can easily get this mixed up (see the
	discussion in our paper on the challenges).;
*---Our model fit considers w_abc, etc., as fixed effects, but Lu & Ades consider them
	as random effects. Our estimates of the terms are thus not shrunken, and can be 
	fairly different from the random-effect terms. In this example, the conclusion is the
	same for our model and the one by Lu & Ades: w_ terms are NOT significant. Thus,
	little evidence of inconsistency.;
*---We primarily give the code for this model as a starting point for those who wish
	to pursue this analytical approach.;
proc glimmix data=a maxopt=100 method=quad;
title2 'Lu & Ades baseline contrasts model, with inconsistency w_ terms (fixed eff.)';
title3 'Use of frequentist conditional model, with random u~_ik';
class study trt;
model m/n = study  x_ab x_ac x_ad   w_abc w_abd w_acd
	/ ddfm=none htype=3 solution ;
random x_ab x_ac x_ad / sub=study type=lin(1) ldata=L3;
estimate 'd_BC' x_ac 1 x_ab -1;
estimate 'd_BD' x_ad 1 x_ab -1;
estimate 'd_CD' x_ad 1 x_ac -1;
run;
*/


**********************************************************************************************;
**********************************************************************************************;

/* 	NOW CODE DATA WITH CONTRAST INDICATORS, ETC., AND THEN CONSTRUCT A FILE WITH
	SUMMARY DATA FOR A SECOND-STAGE ANALYSIS. A _LOT_ OF PROCESSING INVOLVED.  */

*---Data file aa has much in common with a, but some of the coding is different. So, instead
	of trying to make sure the coding was all the same, the two files are kept separate. This will
	be the starting point for making summary data -- either log odds (for the unconditional
	model) or log odds RATIOS (for the conditional model).;

data aa;  set smoking;	*---Code data for study type, and get contrasts for conditional model.;
case = _n_;
trt2=trt; /*needed to get baseline contrasts in Lu et al 2011*/

if trt='a' then trt='za';  /*make a the last category 
                            to get same restriction on 
                            basic effects as Lu&Andes*/
/*group studies according to set of treatments tested*/
if study in (1) then group=1;
if study in (2) then group=2;
if study in (3,4,5) then group=3;
if study in (6,7,8,9,10,11,12,13,14,15,16,17,18,19) then group=4;
if study in (20) then group=5;
if study in (21) then group=6;
if study in (22) then group=7;
if study in (23,24) then group=8;
if m=0 then do; m=m+0.5; n=n+1; end;
baseline='a';
if study in (2,21,22) then baseline='b';
if study in (23,24) then baseline='c';
d_AB=0;
d_AC=0;
d_AD=0;
if baseline='a' then do;
  if trt2='a' then trt2='za';
  if trt2='b' then d_AB=1;
  if trt2='c' then d_AC=1;
  if trt2='d' then d_AD=1;
end;
if baseline='b' then do;
  if trt2='a' then d_AB=-1;
  if trt2='b' then trt2='zb';
  if trt2='c' then do; d_AC=1; d_AB=-1; end;
  if trt2='d' then do; d_AD=1; d_AB=-1; end;
end;
if baseline='c' then do;
  if trt2='a' then d_AC=-1;
  if trt2='b' then do; d_AC=-1; d_AB=1; end;
  if trt2='c' then trt2='zc';
  if trt2='d' then do; d_AD=1; d_AC=-1; end;
end;
run;


/*	Go through the process of getting estimated effect size summaries (and SEs) for each treatment
	of each study (trial). Then, assemble everything together in different files, including
	a covariance matrix file for the within-study contrasts with baseline, and a (scaled) 
	among-study covariance matrix (also for the contrasts with baseline). Then use PROC MIXED 
	for the analysis of the study-summary contrasts with baseline.

	The many intermediate steps are customized for THIS data set. 

	First shown is the (much easier, and equivalent [with REML]) direct 2-way (UNCONDITIONAL) 
	mixed model analysis of the summary data (_not_ using the within-study contrasts).
*/
run;

*---UNCONDITIONAL MODEL ANALYSIS WITH SUMMARY DATA:
	Start by getting the summary effect size for each treatment and study. For the unconditional mixed
	model and this example, this is the log odds (_not_ log odds ratio) for each treatment.
	Simple way to get these is to use GENMOD for each data point.;

*ods listing close;					*<--suppress a lot of output that is not needed;
ods output ParameterEstimates=logodds;
proc genmod data=aa;				*<--get the log odds for each treatment of each study;
title2 'intermediate results for later 2nd stage analysis';
model m/n = / link=logit dist=bin;
by case;							*<--each record is a case, so a log-odds for each record;
run;
*ods listing;						*<--turn listing of output back on;

data logodds;
set logodds;
if Parameter='Intercept';
logodds=estimate;					*<--this is the response variable in the 2nd stage;
weight=1/stderr**2;					*<--weight is the inverse of the within-study variance;
run;

data bb;
merge aa logodds;
by case;
run;

proc print data=bb;
title2 'summary data for 2nd stage meta-analysis (for unconditional mixed model)';
var study trt logodds weight;
run;

*---2-way (UNCONDITIONAL) mixed model on SUMMARY data. Analysis here based on log odds
	for each treatment of each study (not log odds ratio). One specifies the fixed 
	within-study variances by a combination of the weight statement (where 
	weight is 1/(variance of the log odds) for each treatment and study), and
	a fixed value of 1 for the "residual" (done using options on the PARMS
	statement. Although the output says that the "residual" is 1, it really is
	a different value for each study (inverse of the weight).
	One can have a random study*trt effect for the heterogeneity. We use the
	homogeneous case here (same variance for each treatment).;
	
proc mixed data=bb covtest;		*<--Analysis based on estimated log odds, _not_ log odds ratios.;
title2 '2-way (unconditional) mixed model on summaries (log odds), fixed-effects study and trt';
title3 'Homogenous study variance. Hold within-study variances constant using WEIGHT and PARMS';
title4 '(label this "S-1" output, this is identical to "S-2" output (below) based on contrasts';
class study trt;
model logodds = study trt / solution ddfm=kr chisq noint;
random study*trt;
weight weight;			*<--Weight is the inverse of the (est.) WITHIN-study ("sampling") variance of each log odds;
repeated;
parms (1)(1) / hold=2;	*<--Initial estimates of between-study and within-study variances. Must hold second one
							constant at 1 (used in conjunction with WEIGHT). Gives correct within-study variances;
lsmeans trt / pdiff;
estimate 'B-A' trt 1 0 0 -1;	*<---"a" is really last (as "za") --> order: b c d za;
estimate 'C-A' trt 0 1 0 -1;
estimate 'D-A' trt 0 0 1 -1;
run;
run;
*^--Above, the random statement can also be written as "random trt / sub=study".
	The "noint" option is not required.; 


/* Look for inconsistency using fixed effects for study type (group) and type*trt */
proc mixed data=bb covtest;
title2 '(unconditional) mixed model on summaries (log odds) - test for INCONSISTENCY ';
title3 'Homogenous study variance. Hold within-study variances constant using WEIGHT and PARMS';
class study trt group;
model logodds = group study trt group*trt
      /  htype=1 chisq ddfm=kr;
random study*trt;
weight weight;
repeated;
parms (.1 .2 .3 .4)(1) / hold=2;	*<--See comments above for more on the combination of WEIGHT and PARMS (HOLD=);
run;
*^--When using type I tests, put group*trt last, and use this test for inconsistency;

/*FE model*/
proc mixed data=bb;
title2 '2-way (unconditional) mixed model on summaries (log odds) - test for inconsistency';
title3 'FIXED EFFECTS only (no between-study var.). Hold within-study variances constant';
class study trt group;
model logodds=group study trt group*trt
      / solution htype=1 chisq ;
weight weight;
repeated;
parms (1)/hold=1;
run;

**********************************************************************************************;

/*	ANALYSIS OF CONTRASTS WITH BASELINE USING SUMMARY DATA:	
	Get contrasts with baseline of Lu et al (2011) for each study. Need to store the 
	parameter estimates and var-cov matrix of parameter estimates for each study. 
	Then "clean things up" and create the two block-diagonal var-cov matrices (for within _and_ 
	between studies) for contrasts with baseline. 
*/

*ods listing close;				*<--suppress listing to Output for all the intermediate	
									results. One just needs the log odds ratios in a file;
ods output parameterestimates=logOddsRatios covB=covB;
proc genmod data=bb;
title2 'Get contrasts with baseline (log odds RATIOS here) for each study for later analysis';
title3 '(intermediate result for the second stage analysis of est. effect sizes)';
class trt2;
model m/n = trt2 / covb link=logit dist=bin;
by study;
run;
*ods listing;				*<--turn the listing back on;
run;

data logOddsRatios;			*<--Code the model-effects terms to use with the conditional model;
set logOddsRatios;			*<--pay close attention to the baseline for each study (very tricky);
if Parameter='trt2';
trt2=level1;
if DF>0;
baseline='a';
if study in (2,21,22) then baseline='b';
if study in (23,24) then baseline='c';
d_AB=0;
d_AC=0;
d_AD=0;
if baseline='a' then do;
  if trt2='a' then trt2='za';
  if trt2='b' then d_AB=1;
  if trt2='c' then d_AC=1;
  if trt2='d' then d_AD=1;
end;
if baseline='b' then do;
  if trt2='a' then d_AB=-1;
  if trt2='b' then trt2='zb';
  if trt2='c' then do; d_AC=1; d_AB=-1; end;
  if trt2='d' then do; d_AD=1; d_AB=-1; end;
end;
if baseline='c' then do;
  if trt2='a' then d_AC=-1;
  if trt2='b' then do; d_AC=-1; d_AB=1; end;
  if trt2='c' then trt2='zc';
  if trt2='d' then do; d_AD=1; d_AC=-1; end;
end;
run;

data covb;		*<--more processing....;
set covb;
if rowname ne 'Prm1';
drop Prm1;
run;

/*	Now use IML to make the two var-cov matrices for the contrasts with baseline. In previous data step,
	we handled the situation that the baseline varies with study. The within-study var-cov matrix is 
	fixed, in the sense that the elements come directly from the individual-study analyses, and are not
	changed in the second-stage analysis. A so-called homogenous between-study var-cov matrix is 
	used here for the contrasts, with the same variance and covariances across all studies. With this 
	situation, the covariance between a pair of contrasts is half of the variance for any contrast. 
	When a study only has two treatments, the "sub-matrix" for a study is a single element.
*/

proc iml;					
use covb;
read all var _num_ into covb;
print covb;
count=0;
firstStudyID=1;
From=1;
R=j(nrow(covb), nrow(covb), 0);
G=R;
do i=1 to nrow(covb);
  nextStudyID=covb[i,1];
  if i=nrow(covb) then do;
    nextstudyID=nextstudyID+1;
  end;
  if nextStudyID>firststudyID then do;
    to=i-1;
	covbi=covb[from:to,2:to-from+2];
	r[from:to,from:to]=covbi;
	do i=from to to; do j=from to to; if i=j then g[i,j]=2; else g[i,j]=1; end; end;
	print covbi firstStudyID from to;
	firststudyID=nextstudyID;
	from=i;
  end;
  if i=nrow(covb) then do;
    to=i;
	firststudyID=firststudyID-1;
  	covbi=covb[from:to,2:to-from+2];
	r[from:to,from:to]=covbi;
	do i=from to to; do j=from to to; if i=j then g[i,j]=2; else g[i,j]=1; end; end;
	print covbi firstStudyID from to;
  end;
end;
print r g;
create g from g; append from g;
create r from r; append from r;
quit;

data r;
set r;
parm=1;
row=_N_;
run;

data g;
set g;
parm=1;
row=_N_;
run;

*---You can print the block-diagonal matrices, if interested.;
/*
proc print data=r;run;
proc print data=g;run;
*/
proc print data=logOddsRatios;
title2 'summary data file for contrast-based analysis (estimate is the log odds RATIO)';
var study estimate trt2 d_AB d_AC d_AD;
run;

/* Now, the frequentist version of Lu & Ades model, for study SUMMARY contrast data (log odds ratios) */
/* "Estimate" response variable is the log-odds ratio */
proc mixed data=logOddsRatios;
title2 'condtional model fit to summary (contrast with baseline) data (log odds RATIOS)';
title3 'approach uses LDATA= files for within and between-study var-cov matrices';
title4 '(label this "S-2" output, this is identical to "S-1" output (above) based on expected val.)';
class study trt2;
model estimate = d_AB d_AC d_AD / noint solution ddfm=kr chisq;	*<--note the "noint" here (important to keep);
random trt2*study / subject=intercept type=lin(1) ldata=g;		*<--Specify the between-study var-cov matrix;
repeated / subject=intercept type=lin(1) ldata=r;				*<--Specify the within-study var-cov matrix;
contrast 'treatments' d_AB 1, d_AC 1, d_AD 1 / chisq;			*<--Test of "treatment";
parms (1)(1)/hold=2;											*<--Must hold the within-study cov matrix constant;
/*	^Above, a "single" between-study variance parameter (VAR) is estimated (with RANDOM statement). 
	But this translates into a var-cov MATRIX where the parameter (VAR) is multiplied by the elements of the LDATA=G matrix. 
	Here, within each sub-matrix (for a study) in the file G, the diagonals equal 2 and the off-diagonals equal 1 (or there
	are no off-diagonals if there are only two treatments in a study). 	This gives the proper estimated var-cov matrix.
	This is a very difficult or tedious way to get the same result that is obtained above with direct
	analysis (unconditional modeling) of the study summaries (log odds, _not_ odds ratios).
*/

run;


